
#ifndef SRXFRAMEBUFFERCONNECTION_H__
#define SRXFRAMEBUFFERCONNECTION_H__

// Handy IOLog/printf format strings for dealing with types that have a different
// length on LP64.
#if __LP64__
#define UInt32_FORMAT		"%u"
#define UInt32_x_FORMAT		"0x%08x"
#define PhysAddr_FORMAT		"0x%016llx"
#define PhysLen_FORMAT		"%llu"
#define VirtAddr_FORMAT		"0x%016llx"
#define ByteCount_FORMAT	"%llu"
#else
#define UInt32_FORMAT		"%lu"
#define UInt32_x_FORMAT		"0x%08lx"
#define PhysAddr_FORMAT		"0x%08lx"
#define PhysLen_FORMAT		UInt32_FORMAT
#define VirtAddr_FORMAT		"0x%08x"
#define ByteCount_FORMAT	UInt32_FORMAT
#endif

// dispatch methods for DisplayCard
enum {
	kSRXDisplayPlugMonitor,
    kSRXDisplayPlugMonitorAttribute,
	kSRXDisplayunPlugMonitor,
	kSRXDisplayCheckFramebufferState,
	kSRXDisplaySnapshot,
	kSRXDisplayGetModeCount,
	kSRXDisplayGetModeInfo,
	kSRXDisplayGetCursorState,
	kSRXDisplayGetCursorResolution,
	kSRXDisplayEnableCursorEvents,
	kSRXDisplayDisableCursorEvents,
    kSRXDisplayEnableClientCursor,
	kNumberOfMethods
};


// Dispatch methods for FB connector (Video driver)
enum SRXFBConnectorCommandCodes {
    kCompressDone,
	kReleaseCompressBuffer,
	kSetCompressionOnOff,
    kSnapshot,
    kRegisterNotification,
    kunRegisterNotification,
    kGetDigitizerRect,
    kGetMaxSrcRect,
    kNumSRXFBCUCMethods
};

// To avoid invisible compiler padding, align fields on 64-bit boundaries when possible
// and make the whole structure's size a multiple of 64 bits.
typedef struct __StructForSnapshot {
    uint64_t		data_pointer; // Use C99 types to ensure desired size without unexpected truncation of 64-bit quantities.
    uint64_t		data_length;
    uint32_t		parameter1;
    uint32_t		__pad;
} StructForSnapshot;

typedef struct __ResultsForSnapshot {
    uint64_t		results1;
} ResultsForSnapshot;

enum SRXDisplayEventType
{
    kSRXDisplayCursorEvent,
    kSRXDisplayVideoEvent
};

// Cursor events
enum SRXDisplayCursorEvent
{
	kSRXDisplayCursorStateChanged,
	kSRXDisplayCursorImageChanged
};
// Video event
enum SRXDisplayVideoEvent
{
    kSRXDisplayVideoCompressDone
};

// Driver mode info
typedef struct {
	char name[32];
	unsigned int width;
	unsigned int height;
} SRXDisplayModeInfo;
// Attributes of heads of display card
enum {
    kSRXDisplayMain	= (1 << 0),
    kSRXDisplayMirror			= (1 << 1)
};

// class SRXDisplayCard provides service of virtual display card and framebuffer
#define classSRXDisplayCard "com_splashtop_driver_SRXDisplayCard"
#define classSRXFBC "com_splashtop_driver_SRXFBConnector"
//these are used as return values in state functions
#define FBufEnabled 1
#define FBufDisabled 0

// Display card related api
io_service_t SRXDisplayCardFindService(void);
IOReturn SRXDisplayCardPlugMonitor(io_connect_t connect, int mode);
IOReturn SRXDisplayCardPlugMonitorAttribute(io_connect_t connect, int mode, UInt32 Atts);

IOReturn SRXDisplayCardunPlugMonitor(io_connect_t connect);

// Frame buffer related api
// if Framebuffer is connected, return success. if not, return offline
int SRXDisplayCardCheckFramebufferState(io_connect_t connect);
unsigned char *SRXDisplayCardMapFramebuffer(io_connect_t connect, unsigned int *size);
void SRXDisplayCardUnmapFramebuffer(io_connect_t connect, unsigned char *buf);
int SRXDisplayCardSnapshot(io_connect_t connect);
// Get global setting of display card
int SRXDisplayCardGetModeCount(io_connect_t connect);
kern_return_t SRXDisplayCardGetModeInfo(io_connect_t connect, int mode, SRXDisplayModeInfo *info);

// Cursor related api
// Display card service wil draw cursor by default. If client wants to draw cursor by itself, use the methods to disable drawing by display card
// useless for dynamically changing setting of client cursor
bool SRXDisplayCardEnableClientCursor(io_connect_t connect);
unsigned char *SRXDisplayCardMapCursor(io_connect_t connect, unsigned int *size, int *width, int *height);
void SRXDisplayCardUnmapCursor(io_connect_t connect, unsigned char *buf);
void SRXDisplayCardGetCursorState(io_connect_t connect, int *x, int *y, bool *visible);
bool SRXDisplayCardEnableCursorEvents(io_connect_t connect, mach_port_t recallport, void * callback, void *reference);
bool SRXDisplayCardDisableCursorEvents(io_connect_t connect);

// Video service related api
io_service_t SRXFBCFindVideoService(void);
io_service_t SRXFBCFindVideoServiceAttribute(UInt32 Atts);
IOReturn SRXFBCGetDeviceName(const io_service_t inObj, char outDeviceName[]);
IOReturn SRXFBCGetMaxSrcRect(io_connect_t connect, Rect* rect);
IOReturn SRXFBCGetDigitizerRect(io_connect_t connect, Rect* rect);
IOReturn SRXFBCSetCompressionOnOff(io_connect_t connect, bool newState);
IOReturn SRXFBCCompressDone(io_connect_t connect, UInt8* queuedFrameCount, Ptr* theData, long* dataSize );
IOReturn SRXFBCReleaseCompressBuffer(io_connect_t connect );
IOReturn SRXFBCSnapshot(io_connect_t connect,uintptr_t theBufffer, long BufferSize);

BytePtr SRXFBCMapFramebuffer(io_connect_t connect, unsigned int *size);
void SRXFBCUnmapFramebuffer(io_connect_t connect, BytePtr buf);

// Register notification of compress done
IOReturn SRXFBCRegisterNotification(io_connect_t connect, mach_port_t recallport, void * callback, void *reference);
IOReturn SRXFBCunRegisterNotification(io_connect_t connect);


#endif