#!/bin/bash

function vercomp () {
    if [[ $1 == $2 ]]
    then
        return 0
    fi
    local IFS=.
    local i ver1=($1) ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++))
    do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++))
    do
        if [[ -z ${ver2[i]} ]]
        then
            # fill empty fields in ver2 with zeros
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]}))
        then
            return 1
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]}))
        then
            return 2
        fi
    done
    return 0
}

function loadAgentSetting() {
    _PID="$1"
    _UID="$2"
    PLISTFILE="$3"

    MAJOR_MAC_VERSION=$(sw_vers -productVersion | awk -F '.' '{print $1 "." $2}')
    vercomp "${MAJOR_MAC_VERSION}" "10.10"
    ret=$?
    if [[ $ret -eq 1 ]]; then
        echo "launchctl bootout gui/${_UID} \"${PLISTFILE}\"" | sh
        echo "launchctl bootstrap gui/${_UID} \"${PLISTFILE}\"" | sh
    elif [[ $ret -eq 0 ]]; then
        echo "launchctl bsexec \"${_PID}\" chroot -u \"${_UID}\" / launchctl unload -w \"${PLISTFILE}\" 2>&1" | sh
        echo "launchctl bsexec \"${_PID}\" chroot -u \"${_UID}\" / launchctl load -w -S Aqua \"${PLISTFILE}\" 2>/dev/null" | sh
    fi
}

CURRPATH="$(dirname "$0")"
CURRUID="$1"
PLISTFILE="$2"
AGENTPATH="$3"
PRODUCTCODE="$4"

service_target="$(basename \"${PLISTFILE}\")"

MAJOR_MAC_VERSION=$(sw_vers -productVersion | awk -F '.' '{print $1 "." $2}')
for pid_uid in $(ps -axo pid,uid,args | grep -i "[l]oginwindow.app" | awk '{print $1 "," $2}'); do
    pid=$(echo $pid_uid | cut -d, -f1)
    uid=$(echo $pid_uid | cut -d, -f2)
    
    if [ "${uid}" == "0" ]; then
        continue
    fi
    
    if [ "${uid}" == "${CURRUID}" ]; then
        continue
    fi
    
    vercomp "${MAJOR_MAC_VERSION}" "10.9"
    ret=$?
    if [[ $ret -eq 1 ]]; then
        loadAgentSetting "${pid}" "${uid}" "${PLISTFILE}"
    else
        #echo "launchctl bsexec \"$pid\" chroot -u \"$uid\" / launchctl unload \"${PLISTFILE}\"" | sh
    	#echo "launchctl bsexec \"$pid\" chroot -u \"$uid\" / launchctl load -S Aqua \"${PLISTFILE}\"" | sh
        #echo "launchctl bsexec \"$pid\" chroot -u \"$uid\" / \"${CURRPATH}/watchUserLoginLogout.sh\" \"open -a \\\"/Users/Shared/Splashtop SOS Agent/Splashtop SOS.app\\\"\" \"\"" | sh
        #echo "launchctl bsexec \"$pid\" chroot -u \"$uid\" / \"${CURRPATH}/watchUserLoginLogout.sh\" \"${uid}\" \"open -a \\\"/Users/Shared/Splashtop SOS Agent/Splashtop SOS.app\\\"\""
        echo "launchctl bsexec \"$pid\" chroot -u \"$uid\" / \"${CURRPATH}/watchUserLoginLogout.sh\" \"${uid}\" \"\\\"${AGENTPATH}/Contents/MacOS/SRSOSHelper\\\" \\\"${uid}\\\" \\\"\\\" \\\"\\\" \\\"\\\" \\\"${PRODUCTCODE}\\\" runAgent\""
        
        #echo " -u \"$uid\" / \"${CURRPATH}/watchUserLoginLogout.sh\" \"${uid}\" \"open -a \\\"/Users/Shared/Splashtop SOS Agent/Splashtop SOS.app\\\"\""
    fi
done


#echo "/usr/bin/sudo -u \"$(stat -f '%Su' /dev/console)\" /bin/launchctl unload -w \"${PLISTFILE}\" 2>&1" | sh
#echo "/usr/bin/sudo -u \"$(stat -f '%Su' /dev/console)\" /bin/launchctl load -w \"${PLISTFILE}\" 2>/dev/null; /usr/bin/sudo -u \"$(stat -f '%Su' /dev/console)\" /bin/launchctl kill SIGTERM ${service_target%.*}" | sh
echo "/usr/bin/sudo -u \"$(stat -f '%Su' /dev/console)\" /bin/launchctl load -w \"${PLISTFILE}\" 2>/dev/null" | sh
